VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsEvents"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian. S.Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Notify-events
Public Event ElementAdded(ByVal Index As Long)
Public Event ElementRemoved(ByVal Index As Long)

' The array containing all the events
Private aEvents() As EventData

' The amount of events
Public EventCount As Long
Public SegmentSize As Long

' Used when processing what event to execute
Public ElementProcessor As String
Public EnableProcessor As Boolean

' Used to save the mouse detail
Public MouseUpDetail As Long
Public MouseDownDetail As Long

' Used to hold the real amount of elements
Private pTotalCount As Long

' Add a event
Public Function AddEvent(EventType As EventType, Optional EventPressed As Boolean, Optional EventPosX As Long, Optional EventPosY As Long, Optional EventData As Long)

    ' See if we need to allocate more memory
    If EventCount >= pTotalCount Then
    
        ' Increase by segment size
        pTotalCount = pTotalCount + SegmentSize

        ' Increase the array
        ReDim Preserve aEvents(pTotalCount)
        
    End If

    ' Set the different properties
    With aEvents(EventCount)
        .EventTime = GetTickCount
        .EventType = EventType
        .EventPos.x = EventPosX
        .EventPos.y = EventPosY
        .EventData = EventData
        .EventPressed = EventPressed
    End With

    ' Return the index of the event which was created
    AddEvent = EventCount

    ' Increase the index
    EventCount = EventCount + 1
    
    ' A event has been added
    RaiseEvent ElementAdded(EventCount - 1)

End Function

Public Sub RemoveEvent(ByVal Index As Long)
    
    Dim Tell As Long

    ' Firstly, check whether or not this actually is an index
    If IsEvent(Index) Then
    
        ' Decrease the index
        EventCount = EventCount - 1
    
        ' Move all elements above it one step down
        For Tell = Index + 1 To EventCount
            LSet aEvents(Tell - 1) = aEvents(Tell)
        Next

        ' Resize array
        If EventCount < 1 Then
            ' No more elements exists, just erase it
            Erase aEvents
        Else
            If EventCount < pTotalCount - SegmentSize Then
                
                ' Decrease the real size
                pTotalCount = pTotalCount - SegmentSize
                
                ' Make sure the segment size is valid
                If pTotalCount < SegmentSize Then pTotalCount = SegmentSize
                 
                ' Resize the array to fit the amount of elements
                ReDim Preserve aEvents(pTotalCount)
                
            End If
        End If
        
        ' A event has been removed
        RaiseEvent ElementRemoved(Index)
        
    End If

End Sub

Public Sub LoadFile(sFile As String)

    Dim Free As Long
    
    ' Get a free handle
    Free = FreeFile
    
    ' Open the given file
    Open sFile For Binary As #Free
    
        ' Retrieve the six first variables
        Get #Free, , pTotalCount
        Get #Free, , EventCount
        Get #Free, , EnableProcessor
        Get #Free, , ElementProcessor
        Get #Free, , MouseUpDetail
        Get #Free, , MouseDownDetail
        
        ' Allocate the arrya
        ReDim aEvents(pTotalCount)
        
        ' Load the data
        Get #Free, , aEvents
        
    ' Close the file
    Close #Free

End Sub

Public Sub SaveFile(sFile As String)

    Dim Free As Long
    
    ' Get a free handle
    Free = FreeFile
    
    ' Make sure the file dosen't exists
    If Dir(sFile) <> "" Then
        Kill sFile
    End If
    
    ' Open the given file
    Open sFile For Binary As #Free
    
        ' Save all the variables
        Put #Free, , pTotalCount
        Put #Free, , EventCount
        Put #Free, , EnableProcessor
        Put #Free, , ElementProcessor
        Put #Free, , MouseUpDetail
        Put #Free, , MouseDownDetail
        Put #Free, , aEvents
        
    ' Close the file
    Close #Free

End Sub

Public Function IsEvent(ByVal Index As Long) As Boolean

    ' Return whether or not ths index is valid
    IsEvent = CBool(Index >= 0 And Index < EventCount)

End Function

Public Property Get EventType(ByVal Index As Long) As EventType

    ' Verify that the index exists
    If IsEvent(Index) Then
        EventType = aEvents(Index).EventType
    End If

End Property

Public Property Let EventType(ByVal Index As Long, ByVal vNewValue As EventType)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventType = vNewValue
    End If

End Property

Public Property Get EventPosX(ByVal Index As Long) As Long

    ' As the above
    If IsEvent(Index) Then
        EventPosX = aEvents(Index).EventPos.x
    End If

End Property

Public Property Let EventPosX(ByVal Index As Long, ByVal vNewValue As Long)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventPos.x = vNewValue
    End If

End Property

Public Property Get EventPosY(ByVal Index As Long) As Long

    ' As the above
    If IsEvent(Index) Then
        EventPosY = aEvents(Index).EventPos.y
    End If

End Property

Public Property Let EventPosY(ByVal Index As Long, ByVal vNewValue As Long)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventPos.y = vNewValue
    End If

End Property

Public Property Get EventData(ByVal Index As Long) As Long

    ' As the above
    If IsEvent(Index) Then
        EventData = aEvents(Index).EventData
    End If

End Property

Public Property Let EventData(ByVal Index As Long, ByVal vNewValue As Long)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventData = vNewValue
    End If

End Property

Public Property Get EventTime(ByVal Index As Long) As Long

    ' As the above
    If IsEvent(Index) Then
        EventTime = aEvents(Index).EventTime
    End If

End Property

Public Property Let EventTime(ByVal Index As Long, ByVal vNewValue As Long)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventTime = vNewValue
    End If

End Property

Public Property Get EventPressed(ByVal Index As Long) As Boolean

    ' As the above
    If IsEvent(Index) Then
        EventPressed = aEvents(Index).EventPressed
    End If

End Property

Public Property Let EventPressed(ByVal Index As Long, ByVal vNewValue As Boolean)

    ' As the above
    If IsEvent(Index) Then
        aEvents(Index).EventPressed = vNewValue
    End If

End Property

Public Function CloneEvents() As clsEvents

    Dim Tell As Long

    ' Firstly, create a new class
    Set CloneEvents = New clsEvents
    
    ' Add all events (if any)
    If EventCount > 0 Then
        CloneEvents.CopyArray aEvents
    End If
    
    ' Set the other properties
    CloneEvents.ElementProcessor = ElementProcessor
    CloneEvents.EnableProcessor = EnableProcessor
    CloneEvents.EventCount = EventCount

End Function

Friend Sub CopyArray(refArray() As EventData)

    ' First, allocate the array
    ReDim aEvents(LBound(refArray) To UBound(refArray))
    
    ' Then copy it enterely with this API
    CopyMemory aEvents(0), refArray(0), LenB(aEvents(0)) * (UBound(refArray) - LBound(refArray) + 1)

End Sub

Private Sub Class_Initialize()

    ' Used to optimize appending of elements
    SegmentSize = 32
    MouseUpDetail = 50
    MouseDownDetail = 100

End Sub
