VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsRecorder"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian. S.Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Our primary mean of retrieving mouse/keyboard events
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function SetCursorPos Lib "user32" (ByVal x As Long, ByVal y As Long) As Long
Private Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Long) As Integer
Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

' Used to send input
Private Declare Sub mouse_event Lib "user32.dll" (ByVal dwFlags As Long, ByVal dX As Long, ByVal dY As Long, ByVal cButtons As Long, ByVal dwExtraInfo As Long)
Private Declare Sub keybd_event Lib "user32.dll" (ByVal bVk As Byte, ByVal bScan As Byte, ByVal dwFlags As Long, ByVal dwExtraInfo As Long)

' Informative events
Public Event ExecutingEvent(ByVal Index As Long, Cancel As Boolean)

' The class which all events is written to
Public Events As clsEvents

' The interval which new mouse-positions is updated
Private UpdatePosInterval As Long

' Contains the different states of all the keys
Private intKeys(255) As Integer

' Used in moving the cursor
Private destPos As POINTAPI, tempPos As Speed, mouseSpeed As Speed

' Whether or not the recording should be stopped
Private bExit As Boolean

' To avoid checking shift, control and alt twice, exclude certain keys
Const Key_ExcludeStart = 10
Const Key_ExcludeEnd = 12
Const Key_MouseRelated = 7

' Used to simulate a mouse event
Const MOUSE_LEFTDOWN = &H2
Const MOUSE_MIDDLEDOWN = &H20
Const MOUSE_RIGHTDOWN = &H8
Const MOUSE_XDOWN = &H100
Const MOUSE_MOVE = &H1

Public Sub InitiateRecording(Optional lngInterval As Long = 10)

    Dim intTemp() As Integer, Tell As Long, curPos As POINTAPI, lastPos As POINTAPI
    Dim lngType As Long, lngLast As Long, lngCount As Long
    
    ' Reset variables
    bExit = False
    
    ' Start with the up-state
    UpdatePosInterval = Events.MouseUpDetail
    
    ' Let the first operation start now
    lngLast = GetTickCount
    
    ' Retrieve the current state of keys
    RetrieveList intKeys
    
    ' Then keep repeatedly rechecking them
    Do Until bExit
    
        ' Retrieve the updated list
        RetrieveList intTemp
        
        ' Run through all the keys, ...
        For Tell = LBound(intTemp) To UBound(intTemp)
        
            ' ... checking for differency
            If (intTemp(Tell) And &HF0000000) <> (intKeys(Tell) And &HF0000000) Then
            
                ' Reset variable
                lngType = -1
            
                ' Use the number to find the nature of the key
                Select Case Tell
                Case 0 To Key_MouseRelated ' This is a mouse-specific key-event
                
                    ' Change the position interval
                    ChangeInterval UpdatePosInterval, Tell
                
                    ' See whether or not we need to add a wait event
                    CheckForWait lngLast
                
                    ' Retrieve the current cursor position
                    GetCursorPos curPos
                    
                    ' We're dealing with a mouse event
                    lngType = Event_Mouse
                    
                Case Key_ExcludeStart To Key_ExcludeEnd ' Ignore these
                Case Else ' Normal keyboard event
                
                    ' We're dealing with a keyboard event
                    lngType = Event_KeyBoard
                    
                End Select
                
                ' See if we've actually invoked a real event
                If lngType >= 0 Then
                
                    ' See whether or not we need to add a wait event
                    CheckForWait lngLast

                    ' Add the event to the list
                    Events.AddEvent lngType, (intTemp(Tell) And &HF0000000) <> 0, curPos.x, curPos.y, Tell
                
                    ' Save this position
                    GetCursorPos lastPos
                
                End If
                
            End If
        
        Next
        
        ' Copy to the main list
        CopyList intKeys, intTemp, 0, 255
    
        ' Decrease the count
        lngCount = lngCount - lngInterval
    
        ' See if we are allowed to proceed
        If lngCount <= 0 Then
        
            ' Retrieve the mouse position
            GetCursorPos curPos
    
            ' See if we need to update the mouse position
            If curPos.x <> lastPos.x Or curPos.y <> lastPos.y Then
    
                ' See whether or not we need to add a wait event
                CheckForWait lngLast
    
                ' Add the event to change the position of the mouse
                Events.AddEvent Event_MousePos, , curPos.x, curPos.y
    
                ' Save the current position
                GetCursorPos lastPos
    
                ' Reset count
                lngCount = UpdatePosInterval
    
            End If
    
        End If
    
        ' Wait some time
        Sleep lngInterval
        DoEvents
    
    Loop

End Sub

Private Sub ChangeInterval(refCurInterval, ByVal Key As Long)

    ' See if this is a up or down-event
    Select Case Key
    Case MOUSE_LEFTDOWN, MOUSE_RIGHTDOWN, MOUSE_MIDDLEDOWN, MOUSE_XDOWN
        refCurInterval = Events.MouseDownDetail
    Case Else
        refCurInterval = Events.MouseUpDetail
    End Select

End Sub

Private Sub CheckForWait(lngLast As Long)

    ' See whether or not we need to add a wait event
    If GetTickCount <> lngLast Then
        
        ' Add the difference
        Events.AddEvent Event_Wait, , , , GetTickCount - lngLast
        
        ' Save the current time
        lngLast = GetTickCount
        
    End If

End Sub

Public Sub ReplayEvents(Optional StartIndex As Long = -1, Optional EndIndex As Long = -1)

    Dim Tell As Long, bCancel As Boolean, iKey As Long
    Dim iPressed As Long, dwFlags As Long, uPos As POINTAPI
    
    ' Make sure the numbers are valid
    If Not Events.IsEvent(StartIndex) Then StartIndex = 0
    If Not Events.IsEvent(EndIndex) Then EndIndex = Events.EventCount - 1
    
    ' Reset variables
    bExit = False
    
    ' Start with the up-state
    UpdatePosInterval = Events.MouseUpDetail
    
    ' Go through all events from the given start until the end, ...
    For Tell = StartIndex To EndIndex

        ' See if we are to exit
        If bExit Then
            Exit For
        End If
    
        ' Reset variables
        bCancel = False
    
        ' ... executing it, ...
        RaiseEvent ExecutingEvent(Tell, bCancel)
    
        ' ... if we are allowed to do so
        If Not bCancel Then
        
            ' Get the virtual key
            iKey = Events.EventData(Tell)
            iPressed = IIf(Events.EventPressed(Tell), 1, 2)
        
            ' Process the event upon its type first
            Select Case Events.EventType(Tell)
            Case Event_Wait
            
                ' Simply wait the states amount of time
                Wait iKey
            
            Case Event_Mouse
            
                ' Then add the remaining flags
                Select Case iKey
                Case 1: dwFlags = MOUSE_LEFTDOWN * iPressed
                Case 2: dwFlags = MOUSE_RIGHTDOWN * iPressed
                Case 4: dwFlags = MOUSE_MIDDLEDOWN * iPressed
                Case Else: dwFlags = MOUSE_XDOWN * iPressed
                End Select
                 
                ' Change the position interval
                ChangeInterval UpdatePosInterval, iKey
                 
                ' Don't allow moving the mouse any further
                mouseSpeed.dX = 0: mouseSpeed.dY = 0
                
                ' Move the cursor position
                SetCursorPos Events.EventPosX(Tell), Events.EventPosY(Tell)
                
                ' Make sure the virtual key is valid
                If dwFlags > 0 Then
                    ' Execute the mouse input
                    mouse_event dwFlags, 0, 0, IIf(iKey > 4, iKey - 4, 0), 0&
                End If
            
            Case Event_MousePos
            
                ' Get the position of the cursor
                GetCursorPos uPos
            
                ' Update the move-flags
                destPos.x = Events.EventPosX(Tell)
                destPos.y = Events.EventPosY(Tell)
                tempPos.dX = uPos.x
                tempPos.dY = uPos.y
                
                ' Caculate the move-speed
                mouseSpeed.dX = (destPos.x - tempPos.dX) / (UpdatePosInterval / 10)
                mouseSpeed.dY = (destPos.y - tempPos.dY) / (UpdatePosInterval / 10)
            
            Case Event_KeyBoard
            
                ' Execute the keyboard input
                keybd_event iKey, 0&, IIf(iPressed = 1, 2, 0), 0&
            
            End Select

        End If
    
    Next

End Sub

Public Sub StopProcess()

    ' Tell the above functions to stop
    bExit = True

End Sub

' Begin moving towards a point
Private Sub MousePosition()

    ' Make sure there's any point moving the cursor
    If mouseSpeed.dX = 0 And mouseSpeed.dY = 0 Then
        Exit Sub
    End If

    ' Update the mouse positon (if needed)
    If Not (((tempPos.dX + mouseSpeed.dX > destPos.x) And (tempPos.dX - mouseSpeed.dX < destPos.x)) And _
     ((tempPos.dX + mouseSpeed.dX > destPos.x) And (tempPos.dX - mouseSpeed.dX < destPos.x))) Then
        
        ' Increase the position
        With tempPos
            .dX = .dX + mouseSpeed.dX
            .dY = .dY + mouseSpeed.dY
        End With
        
        ' Change the position
        SetCursorPos CLng(tempPos.dX), CLng(tempPos.dY)
        
    End If

End Sub

' Wait a given amount of time
Private Sub Wait(ByVal bMilliseconds As Long)

    ' Run the below code until we've waited enough
    Do Until bMilliseconds <= 10
    
        ' Decrease the amount of time to wait
        bMilliseconds = bMilliseconds - 10
        
        ' Update mouse position
        MousePosition
        
        ' Wait some time
        Sleep 10
        DoEvents
    
    Loop

    ' If anything is remaining, wait for that too
    If bMilliseconds > 0 Then
        Sleep bMilliseconds
    End If
    
End Sub

' The function which fills an array of virtual keyboard codes with states
Private Sub RetrieveList(refArray() As Integer)

    On Error Resume Next
    Dim Tell As Long
    
    ' Allocate our byte-array to fit more correctly
    ReDim refArray(255)
    
    ' Move through all keyboard-/mouse-codes
    For Tell = 1 To 254
    
        ' Fill each one of them with the correct information
        refArray(Tell) = GetAsyncKeyState(Tell)
    
    Next

End Sub

' Copy a list
Private Sub CopyList(destList() As Integer, sourceList() As Integer, ByVal Start As Long, ByVal Length As Long)

    Dim Tell As Long
    
    ' Then copy all data
    For Tell = Start To Length + Start
        destList(Tell - Start) = sourceList(Tell)
    Next

End Sub

Private Sub Class_Initialize()

    ' Set default values
    UpdatePosInterval = 50

End Sub

Private Sub Class_Terminate()

    ' Stop the recording-loop
    StopProcess

End Sub
