Attribute VB_Name = "modMain"
Option Explicit

' Copyright (C) 2006 Kristian. S.Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Declare Function GetVersion Lib "kernel32" () As Long
Declare Function MapVirtualKey Lib "user32" Alias "MapVirtualKeyA" (ByVal wCode As Long, ByVal wMapType As Long) As Long
Declare Function MapVirtualKeyW Lib "user32" (ByVal wCode As Long, ByVal wMapType As Long) As Long
Declare Function GetKeyNameText Lib "user32" Alias "GetKeyNameTextA" (ByVal lParam As Long, ByVal lpBuffer As String, ByVal nSize As Long) As Long
Declare Function GetKeyNameTextW Lib "user32" (ByVal lParam As Long, ByVal lpBuffer As Long, ByVal nSize As Long) As Long
Declare Function GetTickCount Lib "kernel32" () As Long
Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Declare Function GetClientRect Lib "user32" (ByVal hwnd As Long, lpRect As RECT) As Long
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpDest As Any, lpSource As Any, ByVal nCount As Long)
Declare Sub InitCommonControls Lib "comctl32" ()

' The different types of events
Public Enum EventType
    Event_Wait
    Event_Mouse
    Event_MousePos
    Event_KeyBoard
End Enum

' Used in scrolling
Enum ListViewMessages
    LVM_FIRST = &H1000
    LVM_SCROLL = (LVM_FIRST + 20)
    LVM_GETTOPINDEX = (LVM_FIRST + 39)
    LVM_GETCOUNTPERPAGE = (LVM_FIRST + 40)
End Enum

' Holds information about a rectangle
Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

' Holds information about a position
Type POINTAPI
    x As Long
    y As Long
End Type

' Contains information regarding a moving direction
Type Speed
    dX As Double
    dY As Double
End Type

' The property containing information regarding a certain event
Type EventData
    EventTime As Long
    EventType As Long
    EventPos As POINTAPI
    EventData As Long
    EventPressed As Boolean
End Type

' The class which is used to store the different languages
Public Language As New clsLanguage

' All the different settings (currently only language)
Public Settings As New clsSettings

Public Sub Main()

    ' Load the language
    LoadLanguage
    
    ' Initialize common controls
    InitCommonControls

    ' Show the main form
    frmMain.Show

End Sub

Public Sub LoadLanguage()

    ' Load the XML file containing the language
    Language.LoadXML App.Path & "\Languages\" & Settings.Setting("General", _
     "Language", "English") & ".xml"
     
     ' Update all forms
     Language.UpdateAll Forms

End Sub

Public Function GetKeyboardString(ByRef vKey As KeyCodeConstants, Optional ByRef iShift As ShiftConstants) As String

    Dim lScanCode As Long, b() As Byte, sRet As String, sBuf As String
    Dim lSize As Long, bNt As Boolean, iKeyCode As Integer

    ' User definied "keyboard" codes
    Select Case vKey
    Case 1: GetKeyboardString = "LBUTTON"
    Case 2: GetKeyboardString = "RBUTTON"
    Case 3: GetKeyboardString = "CANCEL"
    Case 4: GetKeyboardString = "MBUTTON"
    Case 5: GetKeyboardString = "XBUTTON1"
    Case 6: GetKeyboardString = "XBUTTON2"
    End Select
    
    ' Don't continue if we've gotten the answer
    If GetKeyboardString <> "" Then
        Exit Function
    End If

    ' Determine if we have Unicode support or not:
    bNt = ((GetVersion() And &H80000000) = 0)
    
    ' Split into shift and key portions:
    iShift = (vKey And &HFF00&) \ &H100&
    vKey = vKey And &HFF&
    
    ' Build the string for the return state:
    sRet = _
     IIf(iShift And vbAltMask, "Alt+", vbNullString) & _
      IIf(iShift And vbCtrlMask, "Ctrl+", vbNullString) & _
       IIf(iShift And vbShiftMask, "Shift+", vbNullString)
    
    ' Translate the virtual-key code into a scan code.
    If (bNt) Then
        lScanCode = MapVirtualKeyW(vKey, 0)
    Else
        lScanCode = MapVirtualKey(vKey, 0)
    End If
    
    ' GetKeyNameText retrieves the name of a key (the scan code must be in bits 16-23):
    lScanCode = lScanCode * &H10000
    If (bNt) Then
        ReDim b(0 To 512) As Byte
        lSize = GetKeyNameTextW(lScanCode, VarPtr(b(0)), 256)
        If (lSize > 0) Then
            sBuf = b
            sRet = sRet & Left$(sBuf, lSize)
        End If
    Else
        sBuf = Space$(256)
        lSize = GetKeyNameText(lScanCode, sBuf, 256)
        sRet = sRet & Left$(sBuf, lSize)
    End If
    
    ' Return the given name of the key
    GetKeyboardString = sRet
      
End Function

' Exits the application
Public Sub ExitApp(Sender As Form)

    Dim Form As Form

    ' Unload all forms
    For Each Form In Forms
        
        ' No point unloading the sender
        If Not (Form Is Sender) Then
            Unload Form
        End If
    
    Next

End Sub

' Ensures the given index is visible. If not, it moves the current selection
' according to the last paramamenter relatively of the selections position
Public Sub EnsureVisible(ByVal hwnd As Long, ByVal Index As Long, Optional newRelativePos As Long = 1)

    Dim iCount As Long, iCurPos As Long, uRect As RECT
    
    ' Firstly, get the amount of elements that fits on a page (and the pos of the sel)
    iCount = SendMessage(hwnd, LVM_GETCOUNTPERPAGE, 0, 0)
    iCurPos = SendMessage(hwnd, LVM_GETTOPINDEX, 0, 0)
    
    ' See if the current index is not visible
    If Index < iCurPos Or Index > iCurPos + iCount Then
    
        ' Get the size of the window
        GetClientRect hwnd, uRect
    
        ' Set the new position
        SendMessage hwnd, LVM_SCROLL, 0, (Index - iCurPos + newRelativePos) * _
         ((uRect.Bottom - uRect.Top) / iCount)
    
    End If

End Sub
